@@ -8,6 +8,7 @@ from dataclasses import dataclass, field
 from sqlglot import Schema, exp, maybe_parse
 from sqlglot.errors import SqlglotError
 from sqlglot.optimizer import Scope, build_scope, find_all_in_scope, normalize_identifiers, qualify
+from sqlglot.optimizer.scope import ScopeType
 
 if t.TYPE_CHECKING:
     from sqlglot.dialects.dialect import DialectType
@@ -28,10 +29,7 @@ class Node:
         yield self
 
         for d in self.downstream:
-            if isinstance(d, Node):
-                yield from d.walk()
-            else:
-                yield d
+            yield from d.walk()
 
     def to_html(self, dialect: DialectType = None, **opts) -> GraphHTML:
         nodes = {}
@@ -71,8 +69,10 @@ def lineage(
     column: str | exp.Column,
     sql: str | exp.Expression,
     schema: t.Optional[t.Dict | Schema] = None,
-    sources: t.Optional[t.Dict[str, str | exp.Query]] = None,
+    sources: t.Optional[t.Mapping[str, str | exp.Query]] = None,
     dialect: DialectType = None,
+    scope: t.Optional[Scope] = None,
+    trim_selects: bool = True,
     **kwargs,
 ) -> Node:
     """Build the lineage graph for a column of a SQL query.
@@ -83,6 +83,8 @@ def lineage(
         schema: The schema of tables.
         sources: A mapping of queries which will be used to continue building lineage.
         dialect: The dialect of input SQL.
+        scope: A pre-created scope to use instead.
+        trim_selects: Whether or not to clean up selects by trimming to only relevant columns.
         **kwargs: Qualification optimizer kwargs.
 
     Returns:
@@ -99,14 +101,15 @@ def lineage(
             dialect=dialect,
         )
 
-    qualified = qualify.qualify(
-        expression,
-        dialect=dialect,
-        schema=schema,
-        **{"validate_qualify_columns": False, "identify": False, **kwargs},  # type: ignore
-    )
+    if not scope:
+        expression = qualify.qualify(
+            expression,
+            dialect=dialect,
+            schema=schema,
+            **{"validate_qualify_columns": False, "identify": False, **kwargs},  # type: ignore
+        )
 
-    scope = build_scope(qualified)
+        scope = build_scope(expression)
 
     if not scope:
         raise SqlglotError("Cannot build lineage, sql must be SELECT")
@@ -114,7 +117,7 @@ def lineage(
     if not any(select.alias_or_name == column for select in scope.expression.selects):
         raise SqlglotError(f"Cannot find column '{column}' in query.")
 
-    return to_node(column, scope, dialect)
+    return to_node(column, scope, dialect, trim_selects=trim_selects)
 
 
 def to_node(
@@ -125,13 +128,8 @@ def to_node(
     upstream: t.Optional[Node] = None,
     source_name: t.Optional[str] = None,
     reference_node_name: t.Optional[str] = None,
+    trim_selects: bool = True,
 ) -> Node:
-    source_names = {
-        dt.alias: dt.comments[0].split()[1]
-        for dt in scope.derived_tables
-        if dt.comments and dt.comments[0].startswith("source: ")
-    }
-
     # Find the specific select clause that is the source of the column we want.
     # This can either be a specific, named select or a generic `*` clause.
     select = (
@@ -143,6 +141,17 @@ def to_node(
         )
     )
 
+    if isinstance(scope.expression, exp.Subquery):
+        for source in scope.subquery_scopes:
+            return to_node(
+                column,
+                scope=source,
+                dialect=dialect,
+                upstream=upstream,
+                source_name=source_name,
+                reference_node_name=reference_node_name,
+                trim_selects=trim_selects,
+            )
     if isinstance(scope.expression, exp.Union):
         upstream = upstream or Node(name="UNION", source=scope.expression, expression=select)
 
@@ -170,11 +179,12 @@ def to_node(
                 upstream=upstream,
                 source_name=source_name,
                 reference_node_name=reference_node_name,
+                trim_selects=trim_selects,
             )
 
         return upstream
 
-    if isinstance(scope.expression, exp.Select):
+    if trim_selects and isinstance(scope.expression, exp.Select):
         # For better ergonomics in our node labels, replace the full select with
         # a version that has only the column we care about.
         #   "x", SELECT x, y FROM foo
@@ -206,7 +216,13 @@ def to_node(
             continue
 
         for name in subquery.named_selects:
-            to_node(name, scope=subquery_scope, dialect=dialect, upstream=node)
+            to_node(
+                name,
+                scope=subquery_scope,
+                dialect=dialect,
+                upstream=node,
+                trim_selects=trim_selects,
+            )
 
     # if the select is a star add all scope sources as downstreams
     if select.is_star:
@@ -221,13 +237,31 @@ def to_node(
     # If the source is a UDTF find columns used in the UTDF to generate the table
     if isinstance(source, exp.UDTF):
         source_columns |= set(source.find_all(exp.Column))
+        derived_tables = [
+            source.expression.parent
+            for source in scope.sources.values()
+            if isinstance(source, Scope) and source.is_derived_table
+        ]
+    else:
+        derived_tables = scope.derived_tables
+
+    source_names = {
+        dt.alias: dt.comments[0].split()[1]
+        for dt in derived_tables
+        if dt.comments and dt.comments[0].startswith("source: ")
+    }
 
     for c in source_columns:
         table = c.table
         source = scope.sources.get(table)
 
         if isinstance(source, Scope):
-            selected_node, _ = scope.selected_sources.get(table, (None, None))
+            reference_node_name = None
+            if source.scope_type == ScopeType.DERIVED_TABLE and table not in source_names:
+                reference_node_name = table
+            elif source.scope_type == ScopeType.CTE:
+                selected_node, _ = scope.selected_sources.get(table, (None, None))
+                reference_node_name = selected_node.name if selected_node else None
             # The table itself came from a more specific scope. Recurse into that one using the unaliased column name.
             to_node(
                 c.name,
@@ -236,7 +270,8 @@ def to_node(
                 scope_name=table,
                 upstream=node,
                 source_name=source_names.get(table) or source_name,
-                reference_node_name=selected_node.name if selected_node else None,
+                reference_node_name=reference_node_name,
+                trim_selects=trim_selects,
             )
         else:
             # The source is not a scope - we've reached the end of the line. At this point, if a source is not found
