@@ -141,6 +141,7 @@ x ILIKE '%y%' ESCAPE '\'
 INTERVAL '1' DAY
 INTERVAL '1' MONTH
 INTERVAL '1' YEAR
+INTERVAL '1' HOUR TO SECOND
 INTERVAL '-1' CURRENT_DATE
 INTERVAL '-31' CAST(GETDATE() AS DATE)
 INTERVAL (1 + 3) DAYS
@@ -149,6 +150,7 @@ INTERVAL '1' DAY * 5
 CASE WHEN TRUE THEN INTERVAL '15' DAYS END
 CASE WHEN TRUE THEN 1 ELSE interval END
 CASE WHEN TRUE THEN 1 ELSE "INTERVAL" END
+SELECT asof FROM x
 SELECT * WHERE interval IS NULL
 SELECT * WHERE NOT interval IS NULL
 SELECT * WHERE INTERVAL "is" > 1
@@ -176,6 +178,7 @@ COUNT(DISTINCT CASE WHEN DATE_TRUNC('ISOWEEK', DATE(time_field)) = DATE_TRUNC('I
 COUNT(a, b)
 x[y - 1]
 CASE WHEN SUM(x) > 3 THEN 1 END OVER (PARTITION BY x)
+ANY(x) OVER (PARTITION BY x)
 SUM(ROW() OVER (PARTITION BY x))
 SUM(ROW() OVER (PARTITION BY x + 1))
 SUM(ROW() OVER (PARTITION BY x AND y))
@@ -361,7 +364,6 @@ SELECT GREATEST(a, b, c) FROM test
 SELECT LAST_VALUE(a) FROM test
 SELECT LAST_VALUE(a) IGNORE NULLS OVER () + 1
 SELECT LN(a) FROM test
-SELECT LOG10(a) FROM test
 SELECT MAX(a) FROM test
 SELECT MIN(a) FROM test
 SELECT POWER(a, 2) FROM test
@@ -476,6 +478,7 @@ SELECT 1 UNION (SELECT 2) ORDER BY x
 SELECT * FROM (((SELECT 1) UNION SELECT 2) ORDER BY x LIMIT 1 OFFSET 1)
 SELECT * FROM ((SELECT 1 AS x) CROSS JOIN (SELECT 2 AS y)) AS z
 ((SELECT 1) EXCEPT (SELECT 2))
+((SELECT 1)) LIMIT 1
 VALUES (1) UNION SELECT * FROM x
 WITH a AS (SELECT 1) SELECT a.* FROM a
 WITH a AS (SELECT 1), b AS (SELECT 2) SELECT a.*, b.* FROM a CROSS JOIN b
@@ -637,6 +640,9 @@ CREATE DATABASE IF NOT EXISTS y
 CREATE PROCEDURE IF NOT EXISTS a.b.c() AS 'DECLARE BEGIN; END'
 CREATE TABLE T3 AS (SELECT DISTINCT A FROM T1 EXCEPT (SELECT A FROM T2) LIMIT 1)
 DESCRIBE x
+DESCRIBE EXTENDED a.b
+DESCRIBE FORMATTED a.b
+DESCRIBE SELECT 1
 DROP INDEX a.b.c
 DROP FUNCTION a.b.c (INT)
 DROP MATERIALIZED VIEW x.y.z
@@ -654,6 +660,7 @@ INSERT OVERWRITE TABLE a.b PARTITION(ds) SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds = 'YYYY-MM-DD') SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds, hour) SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds = 'YYYY-MM-DD', hour = 'hh') SELECT x FROM y
+INSERT INTO a.b PARTITION(DAY = '2024-04-14') (col1, col2) SELECT x FROM y
 DELETE FROM x WHERE y > 1
 DELETE FROM y
 DELETE FROM event USING sales WHERE event.eventid = sales.eventid
@@ -685,6 +692,8 @@ INSERT INTO y (a, b, c) SELECT a, b, c FROM x
 INSERT INTO y (SELECT 1) UNION (SELECT 2)
 INSERT INTO result_table (WITH test AS (SELECT * FROM source_table) SELECT * FROM test)
 INSERT INTO "tests_user" ("username", "first_name", "last_name") VALUES ('fiara', 'Fiara', 'Ironhide') RETURNING "tests_user"."id"
+INSERT INTO t1 (tc1 /* tc1 */, tc2 /* tc2 */) SELECT c1 /* sc1 */, c2 /* sc2 */ FROM t
+INSERT INTO t1 ("tc1" /* tc1 */, "tc2" /* tc2 */) SELECT "c1" /* sc1 */, "c2" /* sc2 */ FROM t
 INSERT OVERWRITE TABLE x IF EXISTS SELECT * FROM y
 INSERT OVERWRITE TABLE a.b IF EXISTS SELECT * FROM y
 INSERT OVERWRITE DIRECTORY 'x' SELECT 1
@@ -831,6 +840,7 @@ SELECT * FROM schema.case
 SELECT * FROM current_date
 SELECT * FROM schema.current_date
 SELECT /*+ SOME_HINT(foo) */ 1
+SELECT /*+ REBALANCE */ * FROM foo
 SELECT * FROM (tbl1 CROSS JOIN (SELECT * FROM tbl2) AS t1)
 /* comment1 */ INSERT INTO x /* comment2 */ VALUES (1, 2, 3)
 /* comment1 */ UPDATE tbl /* comment2 */ SET x = 2 WHERE x < 2
@@ -857,3 +867,6 @@ SELECT truncate
 SELECT only
 TRUNCATE(a, b)
 SELECT enum
+SELECT unlogged
+SELECT name
+SELECT copy
\ No newline at end of file
