@@ -224,16 +224,50 @@ class TestLineage(unittest.TestCase):
             downstream.source.sql(dialect="snowflake"),
             "LATERAL FLATTEN(INPUT => TEST_TABLE.RESULT, OUTER => TRUE) AS FLATTENED(SEQ, KEY, PATH, INDEX, VALUE, THIS)",
         )
-        self.assertEqual(
-            downstream.expression.sql(dialect="snowflake"),
-            "VALUE",
-        )
+        self.assertEqual(downstream.expression.sql(dialect="snowflake"), "VALUE")
         self.assertEqual(len(downstream.downstream), 1)
 
         downstream = downstream.downstream[0]
         self.assertEqual(downstream.name, "TEST_TABLE.RESULT")
         self.assertEqual(downstream.source.sql(dialect="snowflake"), "TEST_TABLE AS TEST_TABLE")
 
+        node = lineage(
+            "FIELD",
+            "SELECT FLATTENED.VALUE:field::text AS FIELD FROM SNOWFLAKE.SCHEMA.MODEL AS MODEL_ALIAS, LATERAL FLATTEN(INPUT => MODEL_ALIAS.A) AS FLATTENED",
+            schema={"SNOWFLAKE": {"SCHEMA": {"TABLE": {"A": "integer"}}}},
+            sources={"SNOWFLAKE.SCHEMA.MODEL": "SELECT A FROM SNOWFLAKE.SCHEMA.TABLE"},
+            dialect="snowflake",
+        )
+        self.assertEqual(node.name, "FIELD")
+
+        downstream = node.downstream[0]
+        self.assertEqual(downstream.name, "FLATTENED.VALUE")
+        self.assertEqual(
+            downstream.source.sql(dialect="snowflake"),
+            "LATERAL FLATTEN(INPUT => MODEL_ALIAS.A) AS FLATTENED(SEQ, KEY, PATH, INDEX, VALUE, THIS)",
+        )
+        self.assertEqual(downstream.expression.sql(dialect="snowflake"), "VALUE")
+        self.assertEqual(len(downstream.downstream), 1)
+
+        downstream = downstream.downstream[0]
+        self.assertEqual(downstream.name, "MODEL_ALIAS.A")
+        self.assertEqual(downstream.source_name, "SNOWFLAKE.SCHEMA.MODEL")
+        self.assertEqual(
+            downstream.source.sql(dialect="snowflake"),
+            "SELECT TABLE.A AS A FROM SNOWFLAKE.SCHEMA.TABLE AS TABLE",
+        )
+        self.assertEqual(downstream.expression.sql(dialect="snowflake"), "TABLE.A AS A")
+        self.assertEqual(len(downstream.downstream), 1)
+
+        downstream = downstream.downstream[0]
+        self.assertEqual(downstream.name, "TABLE.A")
+        self.assertEqual(
+            downstream.source.sql(dialect="snowflake"), "SNOWFLAKE.SCHEMA.TABLE AS TABLE"
+        )
+        self.assertEqual(
+            downstream.expression.sql(dialect="snowflake"), "SNOWFLAKE.SCHEMA.TABLE AS TABLE"
+        )
+
     def test_subquery(self) -> None:
         node = lineage(
             "output",
@@ -266,9 +300,66 @@ class TestLineage(unittest.TestCase):
         self.assertEqual(node.name, "a")
         node = node.downstream[0]
         self.assertEqual(node.name, "cte.a")
+        self.assertEqual(node.reference_node_name, "cte")
         node = node.downstream[0]
         self.assertEqual(node.name, "z.a")
 
+        node = lineage(
+            "a",
+            """
+            WITH foo AS (
+              SELECT
+                1 AS a
+            ), bar AS (
+              (
+                SELECT
+                  a + 1 AS a
+                FROM foo
+              )
+            )
+            (
+              SELECT
+                a + b AS a
+              FROM bar
+              CROSS JOIN (
+                SELECT
+                  2 AS b
+              ) AS baz
+            )
+            """,
+        )
+        self.assertEqual(node.name, "a")
+        self.assertEqual(len(node.downstream), 2)
+        a, b = sorted(node.downstream, key=lambda n: n.name)
+        self.assertEqual(a.name, "bar.a")
+        self.assertEqual(len(a.downstream), 1)
+        self.assertEqual(b.name, "baz.b")
+        self.assertEqual(b.downstream, [])
+
+        node = a.downstream[0]
+        self.assertEqual(node.name, "foo.a")
+
+        # Select from derived table
+        node = lineage(
+            "a",
+            "SELECT a FROM (SELECT a FROM x) subquery",
+        )
+        self.assertEqual(node.name, "a")
+        self.assertEqual(len(node.downstream), 1)
+        node = node.downstream[0]
+        self.assertEqual(node.name, "subquery.a")
+        self.assertEqual(node.reference_node_name, "subquery")
+
+        node = lineage(
+            "a",
+            "SELECT a FROM (SELECT a FROM x)",
+        )
+        self.assertEqual(node.name, "a")
+        self.assertEqual(len(node.downstream), 1)
+        node = node.downstream[0]
+        self.assertEqual(node.name, "_q_0.a")
+        self.assertEqual(node.reference_node_name, "_q_0")
+
     def test_lineage_cte_union(self) -> None:
         query = """
         WITH dataset AS (
@@ -353,3 +444,46 @@ class TestLineage(unittest.TestCase):
 
         with self.assertRaises(sqlglot.errors.SqlglotError):
             lineage('"a"', "WITH x AS (SELECT 1 a) SELECT a FROM x", dialect="snowflake")
+
+    def test_ddl_lineage(self) -> None:
+        sql = """
+        INSERT /*+ HINT1 */
+        INTO target (x, y)
+        SELECT subq.x, subq.y
+        FROM (
+          SELECT /*+ HINT2 */
+            t.x AS x,
+            TO_DATE('2023-12-19', 'YYYY-MM-DD') AS y
+          FROM s.t t
+          WHERE 1 = 1 AND y = TO_DATE('2023-12-19', 'YYYY-MM-DD')
+        ) subq
+        """
+
+        node = lineage("y", sql, dialect="oracle")
+
+        self.assertEqual(node.name, "Y")
+        self.assertEqual(node.expression.sql(dialect="oracle"), "SUBQ.Y AS Y")
+
+        downstream = node.downstream[0]
+        self.assertEqual(downstream.name, "SUBQ.Y")
+        self.assertEqual(
+            downstream.expression.sql(dialect="oracle"), "TO_DATE('2023-12-19', 'YYYY-MM-DD') AS Y"
+        )
+
+    def test_trim(self) -> None:
+        sql = """
+            SELECT a, b, c
+            FROM (select a, b, c from y) z
+        """
+
+        node = lineage("a", sql, trim_selects=False)
+
+        self.assertEqual(node.name, "a")
+        self.assertEqual(
+            node.source.sql(),
+            "SELECT z.a AS a, z.b AS b, z.c AS c FROM (SELECT y.a AS a, y.b AS b, y.c AS c FROM y AS y) AS z",
+        )
+
+        downstream = node.downstream[0]
+        self.assertEqual(downstream.name, "z.a")
+        self.assertEqual(downstream.source.sql(), "SELECT y.a AS a, y.b AS b, y.c AS c FROM y AS y")
